# License : AGPLv3

from rtlsdr import RtlSdr
import numpy as np
import asyncio

from .utils import *

class RtlSdr_NFS32002:

    def __init__(self):
        self.sdr = RtlSdr()
        self.sdr.sample_rate = 1e6
        self.sdr.center_freq = 868.3e6

    def __detectNFS32002Frame(self, samples_array, error_rate):
        nfs32002_timings = [625, 312.5, 312.5, 207.5, 207.5, 500, 500, 250, 250, 250, 250, 500, 500, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 500, 250, 250, 500, 250, 250, 500, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250, 250]
        
        data = np.abs(samples_array)**2
        mean_data = np.mean(data)
        normalized = np.where(data > mean_data, 1, 0)
        bin = normalized[np.where(normalized != 0)[0][0]:]
        bin = np.append([0], bin)

        values, timings = find_runs(bin)
        error_rate_min, error_rate_max = 1-error_rate, 1+error_rate

        detected_frame = False

        i = 0
        while i < len(values):
            # Check the presence of a syncword
            data = True
            if values[i] == 1:
                j = i
                for timing in nfs32002_timings:
                    if (timings[j] >= (timing*error_rate_min) and \
                        timings[j] <= (timing*error_rate_max) and \
                        j < len(values)):
                        j += 1
                    else:
                        data = False
                        break
                if data:
                    detected_frame = True
                    break
            i += 1

        return(detected_frame)

    async def __detectionLoop(self, callback, error_rate):
        samples_array = np.array([])
        stream = self.sdr.stream()
        async for samples in stream:
            samples_array = np.append(samples_array, samples)
       
            if len(samples_array) > 250*200:
                
                try:
                    detected = self.__detectNFS32002Frame(samples_array, error_rate)
                except:
                    detected = False
                # Flush samples array
                samples_array = np.array([])

                if detected:
                    callback()
                    await stream.queue.get()


    def startDetection(self, callback, error_rate = 0.2):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.__detectionLoop(callback, error_rate))



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# The requirements file
with open('requirements.txt') as f:
    required = f.read().splitlines()

# This call to setup() does all the work
setup(
    name="rtlsdr_nfs32002",
    version="0.1",
    description="Réimplémentation du protocole NF S 32002 utilisé par les balises sonores des feux piétons. Permet de détecter le signal d'une télécommande à partir d'un RTL SDR.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/balises-ouistici/rtlsdr_nfs32002",
    author="Jérémy Kalsron",
    author_email="jeremy.kalsron@gmail.com  ",
    license="AGPL-3.0",
    classifiers=[
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=required,
    packages=["rtlsdr_nfs32002"],
    include_package_data=True,
)




# License : AGPLv3

import numpy as np

# Find runs of consecutive items in an array.
# From https://gist.github.com/alimanfoo/c5977e87111abe8127453b21204c1065
def find_runs(x):
    # ensure array
    x = np.asanyarray(x)
    if x.ndim != 1:
        raise ValueError('only 1D array supported')
    n = x.shape[0]

    # handle empty array
    if n == 0:
        return np.array([]), np.array([]), np.array([])

    else:
        # find run starts
        loc_run_start = np.empty(n, dtype=bool)
        loc_run_start[0] = True
        np.not_equal(x[:-1], x[1:], out=loc_run_start[1:])
        run_starts = np.nonzero(loc_run_start)[0]

        # find run values
        run_values = x[loc_run_start]

        # find run lengths
        run_lengths = np.diff(np.append(run_starts, n))

        return run_values, run_lengths

# Convert data to digital frame
def dataToBinary(samples):
    samples = np.abs(samples)**2
    vmax = np.amax(samples)
    threshold = vmax/10 # 0.004
    binaryFrame = []
    for data in samples:
        if data > threshold:
            binaryFrame.append('1')
        else:
            binaryFrame.append('0')
    return binaryFrame